@@ -20,6 +20,7 @@ import struct
 from pymodbus.pdu import ModbusRequest, ModbusResponse, ModbusExceptions
 from pymodbus.client.sync import ModbusTcpClient as ModbusClient
 from pymodbus.bit_read_message import ReadCoilsRequest
+from pymodbus.compat import int2byte, byte2int
 # --------------------------------------------------------------------------- #
 # configure the client logging
 # --------------------------------------------------------------------------- #
@@ -40,15 +41,41 @@ log.setLevel(logging.DEBUG)
 
 
 class CustomModbusResponse(ModbusResponse):
-    pass
+    function_code = 55
+    _rtu_byte_count_pos = 2
+
+    def __init__(self, values=None, **kwargs):
+        ModbusResponse.__init__(self, **kwargs)
+        self.values = values or []
+    
+    def encode(self):
+        """ Encodes response pdu
+
+        :returns: The encoded packet message
+        """
+        result = int2byte(len(self.values) * 2)
+        for register in self.values:
+            result += struct.pack('>H', register)
+        return result
+
+    def decode(self, data):
+        """ Decodes response pdu
+
+        :param data: The packet data to decode
+        """
+        byte_count = byte2int(data[0])
+        self.values = []
+        for i in range(1, byte_count + 1, 2):
+            self.values.append(struct.unpack('>H', data[i:i + 2])[0])
 
 
 class CustomModbusRequest(ModbusRequest):
 
-    function_code = 1
+    function_code = 55
+    _rtu_frame_size = 8
 
-    def __init__(self, address):
-        ModbusRequest.__init__(self)
+    def __init__(self, address=None, **kwargs):
+        ModbusRequest.__init__(self, **kwargs)
         self.address = address
         self.count = 16
 
@@ -74,12 +101,12 @@ class CustomModbusRequest(ModbusRequest):
 
 class Read16CoilsRequest(ReadCoilsRequest):
 
-    def __init__(self, address):
+    def __init__(self, address, **kwargs):
         """ Initializes a new instance
 
         :param address: The address to start reading from
         """
-        ReadCoilsRequest.__init__(self, address, 16)
+        ReadCoilsRequest.__init__(self, address, 16, **kwargs)
 
 # --------------------------------------------------------------------------- #
 # execute the request with your client
@@ -90,7 +117,8 @@ class Read16CoilsRequest(ReadCoilsRequest):
 
 
 if __name__ == "__main__":
-    with ModbusClient('127.0.0.1') as client:
-        request = CustomModbusRequest(0)
+    with ModbusClient(host='localhost', port=5020) as client:
+        client.register(CustomModbusResponse)
+        request = CustomModbusRequest(1, unit=1)
         result = client.execute(request)
-        print(result)
+        print(result.values)
