@@ -14,11 +14,11 @@ the guard construct that is available in python 2.5 and up::
         print result
 """
 # --------------------------------------------------------------------------- #
-# import the various server implementations
+# import the various client implementations
 # --------------------------------------------------------------------------- #
-# from pymodbus.client.sync import ModbusTcpClient as ModbusClient
+from pymodbus.client.sync import ModbusTcpClient as ModbusClient
 # from pymodbus.client.sync import ModbusUdpClient as ModbusClient
-from pymodbus.client.sync import ModbusSerialClient as ModbusClient
+# from pymodbus.client.sync import ModbusSerialClient as ModbusClient
 
 # --------------------------------------------------------------------------- #
 # configure the client logging
@@ -56,6 +56,12 @@ def run_sync_client():
     # * retries - Specify how many retries to allow per transaction (default=3)
     # * retry_on_empty - Is an empty response a retry (default = False)
     # * source_address - Specifies the TCP source address to bind to
+    # * strict - Applicable only for Modbus RTU clients.
+    #            Adheres to modbus protocol for timing restrictions
+    #            (default = True).
+    #            Setting this to False would disable the inter char timeout
+    #            restriction (t1.5) for Modbus RTU
+    #
     #
     # Here is an example of using these options::
     #
@@ -92,18 +98,19 @@ def run_sync_client():
     # are not known to these tests. Furthermore, some use the same memory
     # blocks for the two sets, so a change to one is a change to the other.
     # Keep both of these cases in mind when testing as the following will
-    # _only_ pass with the supplied async modbus server (script supplied).
+    # _only_ pass with the supplied asynchronous modbus server (script supplied).
     # ----------------------------------------------------------------------- #
     log.debug("Write to a Coil and read back")
     rq = client.write_coil(0, True, unit=UNIT)
     rr = client.read_coils(0, 1, unit=UNIT)
     assert(not rq.isError())     # test that we are not an error
+    assert(not rr.isError())     # test that we are not an error
     assert(rr.bits[0] == True)          # test the expected value
 
     log.debug("Write to multiple coils and read back- test 1")
     rq = client.write_coils(1, [True]*8, unit=UNIT)
-    assert(not rq.isError())     # test that we are not an error
     rr = client.read_coils(1, 21, unit=UNIT)
+    assert(not rq.isError())     # test that we are not an error
     assert(not rr.isError())     # test that we are not an error
     resp = [True]*21
 
@@ -118,27 +125,30 @@ def run_sync_client():
     rq = client.write_coils(1, [False]*8, unit=UNIT)
     rr = client.read_coils(1, 8, unit=UNIT)
     assert(not rq.isError())     # test that we are not an error
+    assert(not rr.isError())     # test that we are not an error
     assert(rr.bits == [False]*8)         # test the expected value
 
     log.debug("Read discrete inputs")
     rr = client.read_discrete_inputs(0, 8, unit=UNIT)
-    assert(not rq.isError())     # test that we are not an error
+    assert(not rr.isError())     # test that we are not an error
 
     log.debug("Write to a holding register and read back")
     rq = client.write_register(1, 10, unit=UNIT)
     rr = client.read_holding_registers(1, 1, unit=UNIT)
     assert(not rq.isError())     # test that we are not an error
+    assert(not rr.isError())     # test that we are not an error
     assert(rr.registers[0] == 10)       # test the expected value
 
     log.debug("Write to multiple holding registers and read back")
     rq = client.write_registers(1, [10]*8, unit=UNIT)
     rr = client.read_holding_registers(1, 8, unit=UNIT)
     assert(not rq.isError())     # test that we are not an error
+    assert(not rr.isError())     # test that we are not an error
     assert(rr.registers == [10]*8)      # test the expected value
 
     log.debug("Read input registers")
     rr = client.read_input_registers(1, 8, unit=UNIT)
-    assert(not rq.isError())     # test that we are not an error
+    assert(not rr.isError())     # test that we are not an error
 
     arguments = {
         'read_address':    1,
@@ -150,6 +160,7 @@ def run_sync_client():
     rq = client.readwrite_registers(unit=UNIT, **arguments)
     rr = client.read_holding_registers(1, 8, unit=UNIT)
     assert(not rq.isError())     # test that we are not an error
+    assert(not rr.isError())     # test that we are not an error
     assert(rq.registers == [20]*8)      # test the expected value
     assert(rr.registers == [20]*8)      # test the expected value
 
