@@ -6,6 +6,7 @@ This is a simple web frontend using bottle as the web framework.
 This can be hosted using any wsgi adapter.
 """
 from __future__ import print_function
+from pymodbus.version import version
 import json, inspect
 from bottle import route, request, Bottle
 from bottle import static_file
@@ -222,7 +223,7 @@ def build_application(server):
     """ Helper method to create and initiailze a bottle application
 
     :param server: The modbus server to pull instance data from
-    :returns: An initialied bottle application
+    :returns: An initialized bottle application
     """
     log.info("building web application")
     api = ModbusApiWebApp(server)
@@ -260,7 +261,7 @@ if __name__ == '__main__':
     # ------------------------------------------------------------
     # an example server configuration
     # ------------------------------------------------------------
-    from pymodbus.server.async import ModbusServerFactory
+    from pymodbus.server.asynchronous import ModbusServerFactory
     from pymodbus.constants import Defaults
     from pymodbus.device import ModbusDeviceIdentification
     from pymodbus.datastore import ModbusSequentialDataBlock
@@ -274,10 +275,10 @@ if __name__ == '__main__':
     identity = ModbusDeviceIdentification()
     identity.VendorName  = 'Pymodbus'
     identity.ProductCode = 'PM'
-    identity.VendorUrl   = 'http://github.com/bashwork/pymodbus/'
+    identity.VendorUrl   = 'http://github.com/riptideio/pymodbus/'
     identity.ProductName = 'Pymodbus Server'
     identity.ModelName   = 'Pymodbus Server'
-    identity.MajorMinorRevision = '1.0'
+    identity.MajorMinorRevision = version.short()
 
     # ------------------------------------------------------------
     # initialize the datastore
