@@ -2,6 +2,7 @@ import socket
 import select
 import serial
 import time
+import ssl
 import sys
 from functools import partial
 from pymodbus.constants import Defaults
@@ -13,6 +14,7 @@ from pymodbus.transaction import FifoTransactionManager
 from pymodbus.transaction import DictTransactionManager
 from pymodbus.transaction import ModbusSocketFramer, ModbusBinaryFramer
 from pymodbus.transaction import ModbusAsciiFramer, ModbusRtuFramer
+from pymodbus.transaction import ModbusTlsFramer
 from pymodbus.client.common import ModbusClientMixin
 
 # --------------------------------------------------------------------------- #
@@ -39,12 +41,10 @@ class BaseModbusClient(ModbusClientMixin):
         :param framer: The modbus framer implementation to use
         """
         self.framer = framer
-        if isinstance(self.framer, ModbusSocketFramer):
-            self.transaction = DictTransactionManager(self, **kwargs)
-        else:
-            self.transaction = FifoTransactionManager(self, **kwargs)
+        self.transaction = DictTransactionManager(self, **kwargs)
         self._debug = False
         self._debugfd = None
+        self.broadcast_enable = kwargs.get('broadcast_enable', Defaults.broadcast_enable)
 
     # ----------------------------------------------------------------------- #
     # Client interface
@@ -72,8 +72,9 @@ class BaseModbusClient(ModbusClientMixin):
         )
 
     def send(self, request):
-        _logger.debug("New Transaction state 'SENDING'")
-        self.state = ModbusTransactionState.SENDING
+        if self.state != ModbusTransactionState.RETRYING:
+            _logger.debug("New Transaction state 'SENDING'")
+            self.state = ModbusTransactionState.SENDING
         return self._send(request)
 
     def _send(self, request):
@@ -154,8 +155,16 @@ class BaseModbusClient(ModbusClientMixin):
         try:
             fd.write(hexlify_packets(data))
         except Exception as e:
-            self._logger.debug(hexlify_packets(data))
-            self._logger.exception(e)
+            _logger.debug(hexlify_packets(data))
+            _logger.exception(e)
+
+    def register(self, function):
+        """
+        Registers a function and sub function class with the decoder
+        :param function: Custom function class to register
+        :return:
+        """
+        self.framer.decoder.register(function)
 
     def __str__(self):
         """ Builds a string representation of the connection
@@ -196,12 +205,15 @@ class ModbusTcpClient(BaseModbusClient):
 
         :returns: True if connection succeeded, False otherwise
         """
-        if self.socket: return True
+        if self.socket:
+            return True
         try:
             self.socket = socket.create_connection(
                 (self.host, self.port),
                 timeout=self.timeout,
                 source_address=self.source_address)
+            _logger.debug("Connection to Modbus server established. "
+                          "Socket {}".format(self.socket.getsockname()))
         except socket.error as msg:
             _logger.error('Connection to (%s, %s) '
                           'failed: %s' % (self.host, self.port, msg))
@@ -215,6 +227,15 @@ class ModbusTcpClient(BaseModbusClient):
             self.socket.close()
         self.socket = None
 
+    def _check_read_buffer(self, recv_size=None):
+        time_ = time.time()
+        end = time_ + self.timeout
+        data = None
+        ready = select.select([self.socket], [], [], end - time_)
+        if ready[0]:
+            data = self.socket.recv(1024)
+        return data
+
     def _send(self, request):
         """ Sends data on the underlying socket
 
@@ -223,10 +244,171 @@ class ModbusTcpClient(BaseModbusClient):
         """
         if not self.socket:
             raise ConnectionException(self.__str__())
+        if self.state == ModbusTransactionState.RETRYING:
+            data = self._check_read_buffer()
+            if data:
+                return data
+
         if request:
             return self.socket.send(request)
         return 0
 
+    def _recv(self, size):
+        """ Reads data from the underlying descriptor
+
+        :param size: The number of bytes to read
+        :return: The bytes read if the peer sent a response, or a zero-length
+                 response if no data packets were received from the client at
+                 all.
+        :raises: ConnectionException if the socket is not initialized, or the
+                 peer either has closed the connection before this method is
+                 invoked or closes it before sending any data before timeout.
+        """
+        if not self.socket:
+            raise ConnectionException(self.__str__())
+
+        # socket.recv(size) waits until it gets some data from the host but
+        # not necessarily the entire response that can be fragmented in
+        # many packets.
+        # To avoid the splitted responses to be recognized as invalid
+        # messages and to be discarded, loops socket.recv until full data
+        # is received or timeout is expired.
+        # If timeout expires returns the read data, also if its length is
+        # less than the expected size.
+        self.socket.setblocking(0)
+
+        timeout = self.timeout
+
+        # If size isn't specified read up to 4096 bytes at a time.
+        if size is None:
+            recv_size = 4096
+        else:
+            recv_size = size
+
+        data = []
+        data_length = 0
+        time_ = time.time()
+        end = time_ + timeout
+        while recv_size > 0:
+            ready = select.select([self.socket], [], [], end - time_)
+            if ready[0]:
+                recv_data = self.socket.recv(recv_size)
+                if recv_data == b'':
+                    return self._handle_abrupt_socket_close(
+                        size, data, time.time() - time_)
+                data.append(recv_data)
+                data_length += len(recv_data)
+            time_ = time.time()
+
+            # If size isn't specified continue to read until timeout expires.
+            if size:
+                recv_size = size - data_length
+
+            # Timeout is reduced also if some data has been received in order
+            # to avoid infinite loops when there isn't an expected response
+            # size and the slave sends noisy data continuously.
+            if time_ > end:
+                break
+
+        return b"".join(data)
+
+    def _handle_abrupt_socket_close(self, size, data, duration):
+        """ Handle unexpected socket close by remote end
+
+        Intended to be invoked after determining that the remote end
+        has unexpectedly closed the connection, to clean up and handle
+        the situation appropriately.
+
+        :param size: The number of bytes that was attempted to read
+        :param data: The actual data returned
+        :param duration: Duration from the read was first attempted
+               until it was determined that the remote closed the
+               socket
+        :return: The more than zero bytes read from the remote end
+        :raises: ConnectionException If the remote end didn't send any
+                 data at all before closing the connection.
+        """
+        self.close()
+        readsize = ("read of %s bytes" % size if size
+                    else "unbounded read")
+        msg = ("%s: Connection unexpectedly closed "
+               "%.6f seconds into %s" % (self, duration, readsize))
+        if data:
+            result = b"".join(data)
+            msg += " after returning %s bytes" % len(result)
+            _logger.warning(msg)
+            return result
+        msg += " without response from unit before it closed connection"
+        raise ConnectionException(msg)
+
+    def is_socket_open(self):
+        return True if self.socket is not None else False
+
+    def __str__(self):
+        """ Builds a string representation of the connection
+
+        :returns: The string representation
+        """
+        return "ModbusTcpClient(%s:%s)" % (self.host, self.port)
+
+    def __repr__(self):
+        return (
+            "<{} at {} socket={self.socket}, ipaddr={self.host}, "
+            "port={self.port}, timeout={self.timeout}>"
+        ).format(self.__class__.__name__, hex(id(self)), self=self)
+
+# --------------------------------------------------------------------------- #
+# Modbus TLS Client Transport Implementation
+# --------------------------------------------------------------------------- #
+
+
+class ModbusTlsClient(ModbusTcpClient):
+    """ Implementation of a modbus tls client
+    """
+
+    def __init__(self, host='localhost', port=Defaults.TLSPort, sslctx=None,
+        framer=ModbusTlsFramer, **kwargs):
+        """ Initialize a client instance
+
+        :param host: The host to connect to (default localhost)
+        :param port: The modbus port to connect to (default 802)
+        :param sslctx: The SSLContext to use for TLS (default None and auto create)
+        :param source_address: The source address tuple to bind to (default ('', 0))
+        :param timeout: The timeout to use for this socket (default Defaults.Timeout)
+        :param framer: The modbus framer to use (default ModbusSocketFramer)
+
+        .. note:: The host argument will accept ipv4 and ipv6 hosts
+        """
+        self.sslctx = sslctx
+        if self.sslctx is None:
+            self.sslctx = ssl.create_default_context()
+            # According to MODBUS/TCP Security Protocol Specification, it is
+            # TLSv2 at least
+            self.sslctx.options |= ssl.OP_NO_TLSv1_1
+            self.sslctx.options |= ssl.OP_NO_TLSv1
+            self.sslctx.options |= ssl.OP_NO_SSLv3
+            self.sslctx.options |= ssl.OP_NO_SSLv2
+        ModbusTcpClient.__init__(self, host, port, framer, **kwargs)
+
+    def connect(self):
+        """ Connect to the modbus tls server
+
+        :returns: True if connection succeeded, False otherwise
+        """
+        if self.socket: return True
+        try:
+            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            sock.bind(self.source_address)
+            self.socket = self.sslctx.wrap_socket(sock, server_side=False,
+                                                  server_hostname=self.host)
+            self.socket.settimeout(self.timeout)
+            self.socket.connect((self.host, self.port))
+        except socket.error as msg:
+            _logger.error('Connection to (%s, %s) '
+                          'failed: %s' % (self.host, self.port, msg))
+            self.close()
+        return self.socket is not None
+
     def _recv(self, size):
         """ Reads data from the underlying descriptor
 
@@ -244,8 +426,6 @@ class ModbusTcpClient(BaseModbusClient):
         # is received or timeout is expired.
         # If timeout expires returns the read data, also if its length is
         # less than the expected size.
-        self.socket.setblocking(0)
-
         timeout = self.timeout
 
         # If size isn't specified read 1 byte at a time.
@@ -255,41 +435,38 @@ class ModbusTcpClient(BaseModbusClient):
             recv_size = size
 
         data = b''
-        begin = time.time()
+        time_ = time.time()
+        end = time_ + timeout
         while recv_size > 0:
-            ready = select.select([self.socket], [], [], timeout)
-            if ready[0]:
-                data += self.socket.recv(recv_size)
+            data += self.socket.recv(recv_size)
+            time_ = time.time()
 
             # If size isn't specified continue to read until timeout expires.
             if size:
                 recv_size = size - len(data)
 
             # Timeout is reduced also if some data has been received in order
-            # to avoid infinite loops when there isn't an expected response size
-            # and the slave sends noisy data continuosly.
-            timeout -= time.time() - begin
-            if timeout <= 0:
+            # to avoid infinite loops when there isn't an expected response
+            # size and the slave sends noisy data continuously.
+            if time_ > end:
                 break
 
         return data
 
-    def is_socket_open(self):
-        return True if self.socket is not None else False
-
     def __str__(self):
         """ Builds a string representation of the connection
 
         :returns: The string representation
         """
-        return "ModbusTcpClient(%s:%s)" % (self.host, self.port)
+        return "ModbusTlsClient(%s:%s)" % (self.host, self.port)
 
     def __repr__(self):
         return (
             "<{} at {} socket={self.socket}, ipaddr={self.host}, "
-            "port={self.port}, timeout={self.timeout}>"
+            "port={self.port}, sslctx={self.sslctx}, timeout={self.timeout}>"
         ).format(self.__class__.__name__, hex(id(self)), self=self)
 
+
 # --------------------------------------------------------------------------- #
 # Modbus UDP Client Transport Implementation
 # --------------------------------------------------------------------------- #
@@ -372,7 +549,9 @@ class ModbusUdpClient(BaseModbusClient):
         return self.socket.recvfrom(size)[0]
 
     def is_socket_open(self):
-        return True if self.socket is not None else False
+        if self.socket:
+            return True
+        return self.connect()
 
     def __str__(self):
         """ Builds a string representation of the connection
@@ -415,6 +594,9 @@ class ModbusSerialClient(BaseModbusClient):
         :param parity: Which kind of parity to use
         :param baudrate: The baud rate to use for the serial device
         :param timeout: The timeout between serial requests (default 3s)
+        :param strict:  Use Inter char timeout for baudrates <= 19200 (adhere
+        to modbus standards)
+        :param handle_local_echo: Handle local echo of the USB-to-RS485 adaptor
         """
         self.method = method
         self.socket = None
@@ -427,7 +609,9 @@ class ModbusSerialClient(BaseModbusClient):
         self.parity = kwargs.get('parity',   Defaults.Parity)
         self.baudrate = kwargs.get('baudrate', Defaults.Baudrate)
         self.timeout = kwargs.get('timeout',  Defaults.Timeout)
+        self._strict = kwargs.get("strict", False)
         self.last_frame_end = None
+        self.handle_local_echo = kwargs.get("handle_local_echo", False)
         if self.method == "rtu":
             if self.baudrate > 19200:
                 self.silent_interval = 1.75 / 1000  # ms
@@ -460,21 +644,23 @@ class ModbusSerialClient(BaseModbusClient):
 
         :returns: True if connection succeeded, False otherwise
         """
+        import serial
         if self.socket:
             return True
         try:
-            self.socket = serial.Serial(port=self.port,
+            self.socket = serial.serial_for_url(self.port,
                                         timeout=self.timeout,
                                         bytesize=self.bytesize,
                                         stopbits=self.stopbits,
                                         baudrate=self.baudrate,
                                         parity=self.parity)
+            if self.method == "rtu":
+                if self._strict:
+                    self.socket.interCharTimeout = self.inter_char_timeout
+                self.last_frame_end = None
         except serial.SerialException as msg:
             _logger.error(msg)
             self.close()
-        if self.method == "rtu":
-            self.socket.interCharTimeout = self.inter_char_timeout
-            self.last_frame_end = None
         return self.socket is not None
 
     def close(self):
@@ -512,12 +698,19 @@ class ModbusSerialClient(BaseModbusClient):
                 waitingbytes = self._in_waiting()
                 if waitingbytes:
                     result = self.socket.read(waitingbytes)
+                    if self.state == ModbusTransactionState.RETRYING:
+                        _logger.debug("Sending available data in recv "
+                                      "buffer {}".format(
+                            hexlify_packets(result)))
+                        return result
                     if _logger.isEnabledFor(logging.WARNING):
                         _logger.warning("Cleanup recv buffer before "
                                         "send: " + hexlify_packets(result))
             except NotImplementedError:
                 pass
-
+            if self.state != ModbusTransactionState.SENDING:
+                _logger.debug("New Transaction state 'SENDING'")
+                self.state = ModbusTransactionState.SENDING
             size = self.socket.write(request)
             return size
         return 0
@@ -582,5 +775,5 @@ class ModbusSerialClient(BaseModbusClient):
 
 
 __all__ = [
-    "ModbusTcpClient", "ModbusUdpClient", "ModbusSerialClient"
+    "ModbusTcpClient", "ModbusTlsClient", "ModbusUdpClient", "ModbusSerialClient"
 ]
