@@ -13,9 +13,10 @@ kept as a result of a pre-computed lookup dictionary.
 
 from pymodbus.pdu import IllegalFunctionRequest
 from pymodbus.pdu import ExceptionResponse
+from pymodbus.pdu import ModbusRequest, ModbusResponse
 from pymodbus.pdu import ModbusExceptions as ecode
 from pymodbus.interfaces import IModbusDecoder
-from pymodbus.exceptions import ModbusException
+from pymodbus.exceptions import ModbusException, MessageRegisterException
 from pymodbus.bit_read_message import *
 from pymodbus.bit_write_message import *
 from pymodbus.diag_message import *
@@ -26,21 +27,22 @@ from pymodbus.register_read_message import *
 from pymodbus.register_write_message import *
 from pymodbus.compat import byte2int
 
-#---------------------------------------------------------------------------#
+
+# --------------------------------------------------------------------------- #
 # Logging
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
 import logging
 _logger = logging.getLogger(__name__)
 
 
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
 # Server Decoder
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
 class ServerDecoder(IModbusDecoder):
-    ''' Request Message Factory (Server)
+    """ Request Message Factory (Server)
 
     To add more implemented functions, simply add them to the list
-    '''
+    """
     __function_table = [
             ReadHoldingRegistersRequest,
             ReadDiscreteInputsRequest,
@@ -51,19 +53,15 @@ class ServerDecoder(IModbusDecoder):
             WriteSingleRegisterRequest,
             WriteSingleCoilRequest,
             ReadWriteMultipleRegistersRequest,
-
             DiagnosticStatusRequest,
-
             ReadExceptionStatusRequest,
             GetCommEventCounterRequest,
             GetCommEventLogRequest,
             ReportSlaveIdRequest,
-
             ReadFileRecordRequest,
             WriteFileRecordRequest,
             MaskWriteRegisterRequest,
             ReadFifoQueueRequest,
-
             ReadDeviceInformationRequest,
     ]
     __sub_function_table = [
@@ -84,13 +82,12 @@ class ServerDecoder(IModbusDecoder):
             ReturnIopOverrunCountRequest,
             ClearOverrunCountRequest,
             GetClearModbusPlusRequest,
-
             ReadDeviceInformationRequest,
     ]
 
     def __init__(self):
-        ''' Initializes the client lookup tables
-        '''
+        """ Initializes the client lookup tables
+        """
         functions = set(f.function_code for f in self.__function_table)
         self.__lookup = dict([(f.function_code, f) for f in self.__function_table])
         self.__sub_lookup = dict((f, {}) for f in functions)
@@ -98,11 +95,11 @@ class ServerDecoder(IModbusDecoder):
             self.__sub_lookup[f.function_code][f.sub_function_code] = f
 
     def decode(self, message):
-        ''' Wrapper to decode a request packet
+        """ Wrapper to decode a request packet
 
         :param message: The raw modbus request packet
         :return: The decoded modbus message or None if error
-        '''
+        """
         try:
             return self._helper(message)
         except ModbusException as er:
@@ -110,27 +107,34 @@ class ServerDecoder(IModbusDecoder):
         return None
 
     def lookupPduClass(self, function_code):
-        ''' Use `function_code` to determine the class of the PDU.
+        """ Use `function_code` to determine the class of the PDU.
 
         :param function_code: The function code specified in a frame.
         :returns: The class of the PDU that has a matching `function_code`.
-        '''
+        """
         return self.__lookup.get(function_code, ExceptionResponse)
 
     def _helper(self, data):
-        '''
+        """
         This factory is used to generate the correct request object
         from a valid request packet. This decodes from a list of the
         currently implemented request types.
 
         :param data: The request packet to decode
         :returns: The decoded request or illegal function request object
-        '''
+        """
         function_code = byte2int(data[0])
-        _logger.debug("Factory Request[%d]" % function_code)
         request = self.__lookup.get(function_code, lambda: None)()
         if not request:
+            _logger.debug("Factory Request[%d]" % function_code)
             request = IllegalFunctionRequest(function_code)
+        else:
+            fc_string = "%s: %s" % (
+                str(self.__lookup[function_code]).split('.')[-1].rstrip(
+                    "'>"),
+                function_code
+            )
+            _logger.debug("Factory Request[%s]" % fc_string)
         request.decode(data[1:])
 
         if hasattr(request, 'sub_function_code'):
@@ -139,16 +143,36 @@ class ServerDecoder(IModbusDecoder):
             if subtype: request.__class__ = subtype
 
         return request
+    
+    def register(self, function=None):
+        """
+        Registers a function and sub function class with the decoder
+        :param function: Custom function class to register
+        :return:
+        """
+        if function and not issubclass(function, ModbusRequest):
+            raise MessageRegisterException("'{}' is Not a valid Modbus Message"
+                                           ". Class needs to be derived from "
+                                           "`pymodbus.pdu.ModbusRequest` "
+                                           "".format(
+                function.__class__.__name__
+            ))
+        self.__lookup[function.function_code] = function
+        if hasattr(function, "sub_function_code"):
+            if function.function_code not in self.__sub_lookup:
+                self.__sub_lookup[function.function_code] = dict()
+            self.__sub_lookup[function.function_code][
+                function.sub_function_code] = function
 
 
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
 # Client Decoder
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
 class ClientDecoder(IModbusDecoder):
-    ''' Response Message Factory (Client)
+    """ Response Message Factory (Client)
 
     To add more implemented functions, simply add them to the list
-    '''
+    """
     __function_table = [
             ReadHoldingRegistersResponse,
             ReadDiscreteInputsResponse,
@@ -159,19 +183,15 @@ class ClientDecoder(IModbusDecoder):
             WriteSingleRegisterResponse,
             WriteSingleCoilResponse,
             ReadWriteMultipleRegistersResponse,
-
             DiagnosticStatusResponse,
-
             ReadExceptionStatusResponse,
             GetCommEventCounterResponse,
             GetCommEventLogResponse,
             ReportSlaveIdResponse,
-
             ReadFileRecordResponse,
             WriteFileRecordResponse,
             MaskWriteRegisterResponse,
             ReadFifoQueueResponse,
-
             ReadDeviceInformationResponse,
     ]
     __sub_function_table = [
@@ -192,33 +212,33 @@ class ClientDecoder(IModbusDecoder):
             ReturnIopOverrunCountResponse,
             ClearOverrunCountResponse,
             GetClearModbusPlusResponse,
-
             ReadDeviceInformationResponse,
     ]
 
     def __init__(self):
-        ''' Initializes the client lookup tables
-        '''
+        """ Initializes the client lookup tables
+        """
         functions = set(f.function_code for f in self.__function_table)
-        self.__lookup = dict([(f.function_code, f) for f in self.__function_table])
+        self.__lookup = dict([(f.function_code, f)
+                              for f in self.__function_table])
         self.__sub_lookup = dict((f, {}) for f in functions)
         for f in self.__sub_function_table:
             self.__sub_lookup[f.function_code][f.sub_function_code] = f
 
     def lookupPduClass(self, function_code):
-        ''' Use `function_code` to determine the class of the PDU.
+        """ Use `function_code` to determine the class of the PDU.
 
         :param function_code: The function code specified in a frame.
         :returns: The class of the PDU that has a matching `function_code`.
-        '''
+        """
         return self.__lookup.get(function_code, ExceptionResponse)
 
     def decode(self, message):
-        ''' Wrapper to decode a response packet
+        """ Wrapper to decode a response packet
 
         :param message: The raw packet to decode
         :return: The decoded modbus message or None if error
-        '''
+        """
         try:
             return self._helper(message)
         except ModbusException as er:
@@ -229,14 +249,14 @@ class ClientDecoder(IModbusDecoder):
         return None
 
     def _helper(self, data):
-        '''
+        """
         This factory is used to generate the correct response object
         from a valid response packet. This decodes from a list of the
         currently implemented request types.
 
         :param data: The response packet to decode
         :returns: The decoded request or an exception response object
-        '''
+        """
         fc_string = function_code = byte2int(data[0])
         if function_code in self.__lookup:
             fc_string = "%s: %s" % (
@@ -259,7 +279,34 @@ class ClientDecoder(IModbusDecoder):
 
         return response
 
-#---------------------------------------------------------------------------#
+    def register(self, function=None, sub_function=None, force=False):
+        """
+        Registers a function and sub function class with the decoder
+        :param function: Custom function class to register
+        :param sub_function: Custom sub function class to register
+        :param force: Force update the existing class
+        :return:
+        """
+        if function and not issubclass(function, ModbusResponse):
+            raise MessageRegisterException("'{}' is Not a valid Modbus Message"
+                                           ". Class needs to be derived from "
+                                           "`pymodbus.pdu.ModbusResponse` "
+                                           "".format(
+                function.__class__.__name__
+            ))
+        self.__lookup[function.function_code] = function
+        if hasattr(function, "sub_function_code"):
+            if function.function_code not in self.__sub_lookup:
+                self.__sub_lookup[function.function_code] = dict()
+            self.__sub_lookup[function.function_code][
+                function.sub_function_code] = function
+
+
+# --------------------------------------------------------------------------- #
 # Exported symbols
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
+
+
 __all__ = ['ServerDecoder', 'ClientDecoder']
+
+
