@@ -1,16 +1,10 @@
 import struct
-import socket
 from binascii import b2a_hex, a2b_hex
 
 from pymodbus.exceptions import ModbusIOException
 from pymodbus.utilities import checkLRC, computeLRC
 from pymodbus.framer import ModbusFramer, FRAME_HEADER, BYTE_ORDER
 
-# Python 2 compatibility.
-try:
-    TimeoutError
-except NameError:
-    TimeoutError = socket.timeout
 
 ASCII_FRAME_HEADER = BYTE_ORDER + FRAME_HEADER
 
@@ -162,9 +156,10 @@ class ModbusAsciiFramer(ModbusFramer):
 
         :param data: The new packet data
         :param callback: The function to send results to
-        :param unit: Process if unit id matches, ignore otherwise (could be a \
-        list of unit ids (server) or single unit id(client/server))
+        :param unit: Process if unit id matches, ignore otherwise (could be a
+               list of unit ids (server) or single unit id(client/server))
         :param single: True or False (If True, ignore unit address validation)
+
         """
         if not isinstance(unit, (list, tuple)):
             unit = [unit]
