@@ -152,9 +152,10 @@ class ModbusBinaryFramer(ModbusFramer):
 
         :param data: The new packet data
         :param callback: The function to send results to
-        :param unit: Process if unit id matches, ignore otherwise (could be a \
-        list of unit ids (server) or single unit id(client/server)
+        :param unit: Process if unit id matches, ignore otherwise (could be a
+               list of unit ids (server) or single unit id(client/server)
         :param single: True or False (If True, ignore unit address validation)
+
         """
         self.addToFrame(data)
         if not isinstance(unit, (list, tuple)):
