@@ -118,7 +118,7 @@ class ModbusSocketFramer(ModbusFramer):
         if len(data) > self._hsize:
             tid, pid, length, uid, fcode = struct.unpack(SOCKET_FRAME_HEADER,
                                                          data[0:self._hsize+1])
-            return dict(tid=tid, pid=pid, lenght=length, unit=uid, fcode=fcode)
+            return dict(tid=tid, pid=pid, length=length, unit=uid, fcode=fcode)
         return dict()
 
     def processIncomingPacket(self, data, callback, unit, **kwargs):
@@ -136,8 +136,8 @@ class ModbusSocketFramer(ModbusFramer):
 
         :param data: The new packet data
         :param callback: The function to send results to
-        :param unit: Process if unit id matches, ignore otherwise (could be a \
-         list of unit ids (server) or single unit id(client/server)
+        :param unit: Process if unit id matches, ignore otherwise (could be a
+               list of unit ids (server) or single unit id(client/server)
         :param single: True or False (If True, ignore unit address validation)
         :return:
         """
