@@ -7,7 +7,7 @@ import struct
 from pymodbus.constants import ModbusStatus
 from pymodbus.pdu import ModbusRequest
 from pymodbus.pdu import ModbusResponse
-from pymodbus.device import ModbusControlBlock
+from pymodbus.device import ModbusControlBlock, DeviceInformationFactory
 from pymodbus.compat import byte2int, int2byte
 
 _MCB = ModbusControlBlock()
@@ -360,19 +360,26 @@ class ReportSlaveIdRequest(ModbusRequest):
         pass
 
     def execute(self, context=None):
-        ''' Run a read exeception status request against the store
+        ''' Run a report slave id request against the store
 
         :returns: The populated response
         '''
-        identifier = b'Pymodbus'
-        return ReportSlaveIdResponse(identifier)
+        reportSlaveIdData = None
+        if context:
+            reportSlaveIdData = getattr(context, 'reportSlaveIdData', None)
+        if not reportSlaveIdData:
+            information = DeviceInformationFactory.get(_MCB)
+            identifier = "-".join(information.values()).encode()
+            identifier = identifier or b'Pymodbus'
+            reportSlaveIdData = identifier
+        return ReportSlaveIdResponse(reportSlaveIdData)
 
     def __str__(self):
         ''' Builds a representation of the request
 
         :returns: The string representation of the request
         '''
-        return "ResportSlaveIdRequest(%d)" % self.function_code
+        return "ReportSlaveIdRequest(%d)" % self.function_code
 
 
 class ReportSlaveIdResponse(ModbusResponse):
@@ -428,7 +435,7 @@ class ReportSlaveIdResponse(ModbusResponse):
         :returns: The string representation of the response
         '''
         arguments = (self.function_code, self.identifier, self.status)
-        return "ResportSlaveIdResponse(%s, %s, %s)" % arguments
+        return "ReportSlaveIdResponse(%s, %s, %s)" % arguments
 
 #---------------------------------------------------------------------------#
 # TODO Make these only work on serial
