@@ -5,6 +5,7 @@ Collection of transaction based abstractions
 
 import struct
 import socket
+import time
 from threading import RLock
 from functools import partial
 
@@ -14,6 +15,7 @@ from pymodbus.constants import Defaults
 from pymodbus.framer.ascii_framer import ModbusAsciiFramer
 from pymodbus.framer.rtu_framer import ModbusRtuFramer
 from pymodbus.framer.socket_framer import ModbusSocketFramer
+from pymodbus.framer.tls_framer import ModbusTlsFramer
 from pymodbus.framer.binary_framer import ModbusBinaryFramer
 from pymodbus.utilities import hexlify_packets, ModbusTransactionState
 from pymodbus.compat import iterkeys, byte2int
@@ -37,7 +39,7 @@ _logger = logging.getLogger(__name__)
 # The Global Transaction Manager
 # --------------------------------------------------------------------------- #
 class ModbusTransactionManager(object):
-    """ Impelements a transaction for a manager
+    """ Implements a transaction for a manager
 
     The transaction protocol can be represented by the following pseudo code::
 
@@ -61,8 +63,13 @@ class ModbusTransactionManager(object):
         """
         self.tid = Defaults.TransactionId
         self.client = client
-        self.retry_on_empty = kwargs.get('retry_on_empty', Defaults.RetryOnEmpty)
+        self.backoff = kwargs.get('backoff', Defaults.Backoff) or 0.3
+        self.retry_on_empty = kwargs.get('retry_on_empty',
+                                         Defaults.RetryOnEmpty)
+        self.retry_on_invalid = kwargs.get('retry_on_invalid',
+                                           Defaults.RetryOnInvalid)
         self.retries = kwargs.get('retries', Defaults.Retries) or 1
+        self.reset_socket = kwargs.get('reset_socket', True)
         self._transaction_lock = RLock()
         self._no_response_devices = []
         if client:
@@ -78,6 +85,8 @@ class ModbusTransactionManager(object):
             self.base_adu_size = 7  # start(1)+ Address(2), LRC(2) + end(2)
         elif isinstance(self.client.framer, ModbusBinaryFramer):
             self.base_adu_size = 5  # start(1) + Address(1), CRC(2) + end(1)
+        elif isinstance(self.client.framer, ModbusTlsFramer):
+            self.base_adu_size = 0  # no header and footer
         else:
             self.base_adu_size = -1
 
@@ -91,7 +100,8 @@ class ModbusTransactionManager(object):
         """ Returns the length of the Modbus Exception Response according to
         the type of Framer.
         """
-        if isinstance(self.client.framer, ModbusSocketFramer):
+        if isinstance(self.client.framer, (ModbusSocketFramer,
+                                           ModbusTlsFramer)):
             return self.base_adu_size + 2  # Fcode(1), ExcecptionCode(1)
         elif isinstance(self.client.framer, ModbusAsciiFramer):
             return self.base_adu_size + 4  # Fcode(2), ExcecptionCode(2)
@@ -101,6 +111,25 @@ class ModbusTransactionManager(object):
 
         return None
 
+    def _validate_response(self, request, response, exp_resp_len):
+        """
+        Validate Incoming response against request
+        :param request: Request sent
+        :param response: Response received
+        :param exp_resp_len: Expected response length
+        :return: New transactions state
+        """
+        if not response:
+            return False
+
+        mbap = self.client.framer.decode_data(response)
+        if mbap.get('unit') != request.unit_id or mbap.get('fcode') & 0x7F != request.function_code:
+            return False
+
+        if 'length' in mbap and exp_resp_len:
+            return mbap.get('length') == exp_resp_len
+        return True
+
     def execute(self, request):
         """ Starts the producer to send the next request to
         consumer.write(Frame(request))
@@ -112,84 +141,122 @@ class ModbusTransactionManager(object):
                 )
                 retries = self.retries
                 request.transaction_id = self.getNextTID()
-                _logger.debug("Running transaction %d" % request.transaction_id)
+                _logger.debug("Running transaction "
+                              "{}".format(request.transaction_id))
                 _buffer = hexlify_packets(self.client.framer._buffer)
                 if _buffer:
-                    _logger.debug("Clearing current Frame : - {}".format(_buffer))
+                    _logger.debug("Clearing current Frame "
+                                  ": - {}".format(_buffer))
                     self.client.framer.resetFrame()
-
-                expected_response_length = None
-                if not isinstance(self.client.framer, ModbusSocketFramer):
-                    if hasattr(request, "get_response_pdu_size"):
-                        response_pdu_size = request.get_response_pdu_size()
-                        if isinstance(self.client.framer, ModbusAsciiFramer):
-                            response_pdu_size = response_pdu_size * 2
-                        if response_pdu_size:
-                            expected_response_length = self._calculate_response_length(response_pdu_size)
-                if request.unit_id in self._no_response_devices:
-                    full = True
+                broadcast = (self.client.broadcast_enable
+                             and request.unit_id == 0)
+                if broadcast:
+                    self._transact(request, None, broadcast=True)
+                    response = b'Broadcast write sent - no response expected'
                 else:
-                    full = False
-                c_str = str(self.client)
-                if "modbusudpclient" in c_str.lower().strip():
-                    full = True
-                    if not expected_response_length:
-                        expected_response_length = Defaults.ReadSize
-                response, last_exception = self._transact(request,
-                                                          expected_response_length,
-                                                          full=full
-                                                          )
-                if not response and (
-                        request.unit_id not in self._no_response_devices):
-                    self._no_response_devices.append(request.unit_id)
-                elif request.unit_id in self._no_response_devices and response:
-                    self._no_response_devices.remove(request.unit_id)
-                if not response and self.retry_on_empty and retries:
-                    while retries > 0:
-                        if hasattr(self.client, "state"):
-                            _logger.debug("RESETTING Transaction state to "
-                                          "'IDLE' for retry")
-                            self.client.state = ModbusTransactionState.IDLE
-                        _logger.debug("Retry on empty - {}".format(retries))
-                        response, last_exception = self._transact(
-                            request,
-                            expected_response_length
-                        )
-                        if not response:
-                            retries -= 1
-                            continue
-                        # Remove entry
-                        self._no_response_devices.remove(request.unit_id)
-                        break
-                addTransaction = partial(self.addTransaction,
-                                         tid=request.transaction_id)
-                self.client.framer.processIncomingPacket(response,
-                                                         addTransaction,
-                                                         request.unit_id)
-                response = self.getTransaction(request.transaction_id)
-                if not response:
-                    if len(self.transactions):
-                        response = self.getTransaction(tid=0)
+                    expected_response_length = None
+                    if not isinstance(self.client.framer, ModbusSocketFramer):
+                        if hasattr(request, "get_response_pdu_size"):
+                            response_pdu_size = request.get_response_pdu_size()
+                            if isinstance(self.client.framer, ModbusAsciiFramer):
+                                response_pdu_size = response_pdu_size * 2
+                            if response_pdu_size:
+                                expected_response_length = self._calculate_response_length(response_pdu_size)
+                    if request.unit_id in self._no_response_devices:
+                        full = True
                     else:
-                        last_exception = last_exception or (
-                            "No Response received from the remote unit"
-                            "/Unable to decode response")
-                        response = ModbusIOException(last_exception,
-                                                     request.function_code)
-                if hasattr(self.client, "state"):
-                    _logger.debug("Changing transaction state from "
-                                  "'PROCESSING REPLY' to "
-                                  "'TRANSACTION_COMPLETE'")
-                    self.client.state = (
-                        ModbusTransactionState.TRANSACTION_COMPLETE)
+                        full = False
+                    c_str = str(self.client)
+                    if "modbusudpclient" in c_str.lower().strip():
+                        full = True
+                        if not expected_response_length:
+                            expected_response_length = Defaults.ReadSize
+                    response, last_exception = self._transact(
+                        request,
+                        expected_response_length,
+                        full=full,
+                        broadcast=broadcast
+                    )
+                    while retries > 0:
+                        valid_response = self._validate_response(
+                            request, response, expected_response_length
+                        )
+                        if valid_response:
+                            if request.unit_id in self._no_response_devices and response:
+                                self._no_response_devices.remove(request.unit_id)
+                                _logger.debug("Got response!!!")
+                            break
+                        else:
+                            if not response:
+                                if request.unit_id not in self._no_response_devices:
+                                    self._no_response_devices.append(request.unit_id)
+                                if self.retry_on_empty:
+                                    response, last_exception = self._retry_transaction(retries, "empty", request, expected_response_length, full=full)
+                                    retries -= 1
+                                else:
+                                    # No response received and retries not enabled
+                                    break
+                            else:
+                                if self.retry_on_invalid:
+                                    response, last_exception = self._retry_transaction(retries, "invalid", request, expected_response_length, full=full)
+                                    retries -= 1
+                                else:
+                                    break
+                            # full = False
+                    addTransaction = partial(self.addTransaction,
+                                             tid=request.transaction_id)
+                    self.client.framer.processIncomingPacket(response,
+                                                             addTransaction,
+                                                             request.unit_id)
+                    response = self.getTransaction(request.transaction_id)
+                    if not response:
+                        if len(self.transactions):
+                            response = self.getTransaction(tid=0)
+                        else:
+                            last_exception = last_exception or (
+                                "No Response received from the remote unit"
+                                "/Unable to decode response")
+                            response = ModbusIOException(last_exception,
+                                                         request.function_code)
+                        if self.reset_socket:
+                            self.client.close()
+                    if hasattr(self.client, "state"):
+                        _logger.debug("Changing transaction state from "
+                                      "'PROCESSING REPLY' to "
+                                      "'TRANSACTION_COMPLETE'")
+                        self.client.state = (
+                            ModbusTransactionState.TRANSACTION_COMPLETE)
+
                 return response
             except ModbusIOException as ex:
                 # Handle decode errors in processIncomingPacket method
                 _logger.exception(ex)
                 self.client.state = ModbusTransactionState.TRANSACTION_COMPLETE
+                if self.reset_socket:
+                    self.client.close()
                 return ex
 
-    def _transact(self, packet, response_length, full=False):
+    def _retry_transaction(self, retries, reason,
+                           packet, response_length, full=False):
+        _logger.debug("Retry on {} response - {}".format(reason, retries))
+        _logger.debug("Changing transaction state from "
+                      "'WAITING_FOR_REPLY' to 'RETRYING'")
+        self.client.state = ModbusTransactionState.RETRYING
+        if self.backoff:
+            delay = 2 ** (self.retries - retries) * self.backoff
+            time.sleep(delay)
+            _logger.debug("Sleeping {}".format(delay))
+        self.client.connect()
+        if hasattr(self.client, "_in_waiting"):
+            in_waiting = self.client._in_waiting()
+            if in_waiting:
+                if response_length == in_waiting:
+                    result = self._recv(response_length, full)
+                    return result, None
+        return self._transact(packet, response_length, full=full)
+
+    def _transact(self, packet, response_length,
+                  full=False, broadcast=False):
         """
         Does a Write and Read transaction
         :param packet: packet to be sent
@@ -205,22 +272,40 @@ class ModbusTransactionManager(object):
             if _logger.isEnabledFor(logging.DEBUG):
                 _logger.debug("SEND: " + hexlify_packets(packet))
             size = self._send(packet)
+            if isinstance(size, bytes) and self.client.state == ModbusTransactionState.RETRYING:
+                _logger.debug("Changing transaction state from "
+                              "'RETRYING' to 'PROCESSING REPLY'")
+                self.client.state = ModbusTransactionState.PROCESSING_REPLY
+                return size, None
+            if broadcast:
+                if size:
+                    _logger.debug("Changing transaction state from 'SENDING' "
+                                  "to 'TRANSACTION_COMPLETE'")
+                    self.client.state = ModbusTransactionState.TRANSACTION_COMPLETE
+                return b'', None
             if size:
                 _logger.debug("Changing transaction state from 'SENDING' "
                               "to 'WAITING FOR REPLY'")
                 self.client.state = ModbusTransactionState.WAITING_FOR_REPLY
+            if hasattr(self.client, "handle_local_echo") and self.client.handle_local_echo is True:
+                local_echo_packet = self._recv(size, full)
+                if local_echo_packet != packet:
+                    return b'', "Wrong local echo"
             result = self._recv(response_length, full)
+            # result2 = self._recv(response_length, full)
             if _logger.isEnabledFor(logging.DEBUG):
-                _logger.debug("RECV: " + hexlify_packets(result))
+                    _logger.debug("RECV: " + hexlify_packets(result))
+
         except (socket.error, ModbusIOException,
                 InvalidMessageReceivedException) as msg:
-            self.client.close()
+            if self.reset_socket:
+                self.client.close()
             _logger.debug("Transaction failed. (%s) " % msg)
             last_exception = msg
             result = b''
         return result, last_exception
 
-    def _send(self, packet):
+    def _send(self, packet, retrying=False):
         return self.client.framer.sendPacket(packet)
 
     def _recv(self, expected_response_length, full):
@@ -240,9 +325,10 @@ class ModbusTransactionManager(object):
 
             read_min = self.client.framer.recvPacket(min_size)
             if len(read_min) != min_size:
+                msg_start = "Incomplete message" if read_min else "No response"
                 raise InvalidMessageReceivedException(
-                    "Incomplete message received, expected at least %d bytes "
-                    "(%d received)" % (min_size, len(read_min))
+                    "%s received, expected at least %d bytes "
+                    "(%d received)" % (msg_start, min_size, len(read_min))
                 )
             if read_min:
                 if isinstance(self.client.framer, ModbusSocketFramer):
@@ -277,9 +363,14 @@ class ModbusTransactionManager(object):
         result = read_min + result
         actual = len(result)
         if total is not None and actual != total:
-            _logger.debug("Incomplete message received, "
+            msg_start = "Incomplete message" if actual else "No response"
+            _logger.debug("{} received, "
                           "Expected {} bytes Recieved "
-                          "{} bytes !!!!".format(total, actual))
+                          "{} bytes !!!!".format(msg_start, total, actual))
+        elif actual == 0:
+            # If actual == 0 and total is not None then the above
+            # should be triggered, so total must be None here
+            _logger.debug("No response received to unbounded read !!!!")
         if self.client.state != ModbusTransactionState.PROCESSING_REPLY:
             _logger.debug("Changing transaction state from "
                           "'WAITING FOR REPLY' to 'PROCESSING REPLY'")
@@ -289,7 +380,7 @@ class ModbusTransactionManager(object):
     def addTransaction(self, request, tid=None):
         """ Adds a transaction to the handler
 
-        This holds the requets in case it needs to be resent.
+        This holds the request in case it needs to be resent.
         After being sent, the request is removed.
 
         :param request: The request to hold on to
@@ -443,6 +534,6 @@ class FifoTransactionManager(ModbusTransactionManager):
 __all__ = [
     "FifoTransactionManager",
     "DictTransactionManager",
-    "ModbusSocketFramer", "ModbusRtuFramer",
+    "ModbusSocketFramer", "ModbusTlsFramer", "ModbusRtuFramer",
     "ModbusAsciiFramer", "ModbusBinaryFramer",
 ]
