@@ -1,38 +1,49 @@
 #!/usr/bin/env python
+import contextlib
+import sys
 import unittest
 import pytest
 from pymodbus.compat import IS_PYTHON3, PYTHON_VERSION
 if IS_PYTHON3 and PYTHON_VERSION >= (3, 4):
     from unittest.mock import patch, Mock, MagicMock
     import asyncio
-    from pymodbus.client.async.asyncio import AsyncioModbusSerialClient
+    from pymodbus.client.asynchronous.async_io import ReconnectingAsyncioModbusTlsClient
+    from pymodbus.client.asynchronous.async_io import AsyncioModbusSerialClient
     from serial_asyncio import SerialTransport
 else:
     from mock import patch, Mock, MagicMock
 import platform
 from distutils.version import LooseVersion
 
-from pymodbus.client.async.serial import AsyncModbusSerialClient
-from pymodbus.client.async.tcp import AsyncModbusTCPClient
-from pymodbus.client.async.udp import AsyncModbusUDPClient
+from pymodbus.client.asynchronous.serial import AsyncModbusSerialClient
+from pymodbus.client.asynchronous.tcp import AsyncModbusTCPClient
+from pymodbus.client.asynchronous.tls import AsyncModbusTLSClient
+from pymodbus.client.asynchronous.udp import AsyncModbusUDPClient
 
-from pymodbus.client.async.tornado import AsyncModbusSerialClient as AsyncTornadoModbusSerialClient
-from pymodbus.client.async.tornado import AsyncModbusTCPClient as AsyncTornadoModbusTcpClient
-from pymodbus.client.async.tornado import AsyncModbusUDPClient as AsyncTornadoModbusUdoClient
-from pymodbus.client.async import schedulers
+from pymodbus.client.asynchronous.tornado import AsyncModbusSerialClient as AsyncTornadoModbusSerialClient
+from pymodbus.client.asynchronous.tornado import AsyncModbusTCPClient as AsyncTornadoModbusTcpClient
+from pymodbus.client.asynchronous.tornado import AsyncModbusUDPClient as AsyncTornadoModbusUdoClient
+from pymodbus.client.asynchronous import schedulers
 from pymodbus.factory import ClientDecoder
 from pymodbus.exceptions import ConnectionException
-from pymodbus.transaction import ModbusSocketFramer, ModbusRtuFramer, ModbusAsciiFramer, ModbusBinaryFramer
-from pymodbus.client.async.twisted import ModbusSerClientProtocol
+from pymodbus.transaction import ModbusSocketFramer, ModbusTlsFramer, ModbusRtuFramer, ModbusAsciiFramer, ModbusBinaryFramer
+from pymodbus.client.asynchronous.twisted import ModbusSerClientProtocol
+
+import ssl
 
 IS_DARWIN = platform.system().lower() == "darwin"
+IS_WINDOWS = platform.system().lower() == "windows"
 OSX_SIERRA = LooseVersion("10.12")
 if IS_DARWIN:
     IS_HIGH_SIERRA_OR_ABOVE = LooseVersion(platform.mac_ver()[0])
     SERIAL_PORT = '/dev/ttyp0' if not IS_HIGH_SIERRA_OR_ABOVE else '/dev/ptyp0'
 else:
     IS_HIGH_SIERRA_OR_ABOVE = False
-    SERIAL_PORT = "/dev/ptmx"
+    if IS_WINDOWS:
+        # the use is mocked out
+        SERIAL_PORT = ""
+    else:
+        SERIAL_PORT = "/dev/ptmx"
 
 # ---------------------------------------------------------------------------#
 # Fixture
@@ -43,9 +54,18 @@ def mock_asyncio_gather(coro):
     return coro
 
 
+@contextlib.contextmanager
+def maybe_manage(condition, manager):
+    if condition:
+        with manager as value:
+            yield value
+    else:
+        yield None
+
+
 class TestAsynchronousClient(object):
     """
-    This is the unittest for the pymodbus.client.async module
+    This is the unittest for the pymodbus.client.asynchronous module
     """
 
     # -----------------------------------------------------------------------#
@@ -68,8 +88,8 @@ class TestAsynchronousClient(object):
                                  callback=test_callback,
                                  errback=test_errback)
 
-    @patch("pymodbus.client.async.tornado.IOLoop")
-    @patch("pymodbus.client.async.tornado.IOStream")
+    @patch("pymodbus.client.asynchronous.tornado.IOLoop")
+    @patch("pymodbus.client.asynchronous.tornado.IOStream")
     def testTcpTornadoClient(self, mock_iostream, mock_ioloop):
         """ Test the TCP tornado client client initialize """
         protocol, future = AsyncModbusTCPClient(schedulers.IO_LOOP, framer=ModbusSocketFramer(ClientDecoder()))
@@ -104,12 +124,33 @@ class TestAsynchronousClient(object):
         """
         pytest.skip("TBD")
 
+    # -----------------------------------------------------------------------#
+    # Test TLS Client client
+    # -----------------------------------------------------------------------#
+    @pytest.mark.skipif(not IS_PYTHON3 or PYTHON_VERSION < (3, 4),
+                        reason="requires python3.4 or above")
+    def testTlsAsyncioClient(self):
+        """
+        Test the TLS AsyncIO client
+        """
+        loop, client = AsyncModbusTLSClient(schedulers.ASYNC_IO)
+        assert(isinstance(client, ReconnectingAsyncioModbusTlsClient))
+        assert(isinstance(client.framer, ModbusTlsFramer))
+        assert(isinstance(client.sslctx, ssl.SSLContext))
+        assert(client.port == 802)
+
+        def handle_failure(failure):
+            assert(isinstance(failure.exception(), ConnectionException))
+
+        client.stop()
+        assert(client.host is None)
+
     # -----------------------------------------------------------------------#
     # Test UDP client
     # -----------------------------------------------------------------------#
 
-    @patch("pymodbus.client.async.tornado.IOLoop")
-    @patch("pymodbus.client.async.tornado.IOStream")
+    @patch("pymodbus.client.asynchronous.tornado.IOLoop")
+    @patch("pymodbus.client.asynchronous.tornado.IOStream")
     def testUdpTornadoClient(self, mock_iostream, mock_ioloop):
         """ Test the udp tornado client client initialize """
         protocol, future = AsyncModbusUDPClient(schedulers.IO_LOOP, framer=ModbusSocketFramer(ClientDecoder()))
@@ -150,40 +191,44 @@ class TestAsynchronousClient(object):
     # Test Serial client
     # -----------------------------------------------------------------------#
 
+    @pytest.mark.skipif(
+        sys.platform == 'win32' and platform.python_implementation() == 'PyPy',
+        reason='Twisted serial requires pywin32 which is not compatible with PyPy',
+    )
     @pytest.mark.parametrize("method, framer", [("rtu", ModbusRtuFramer),
                                                 ("socket", ModbusSocketFramer),
                                                 ("binary", ModbusBinaryFramer),
                                                 ("ascii", ModbusAsciiFramer)])
     def testSerialTwistedClient(self, method, framer):
-        """ Test the serial tornado client client initialize """
+        """ Test the serial twisted client client initialize """
         from serial import Serial
         with patch("serial.Serial") as mock_sp:
             from twisted.internet import reactor
             from twisted.internet.serialport import SerialPort
+            with maybe_manage(sys.platform == 'win32', patch.object(SerialPort, "_finishPortSetup")):
+                with patch('twisted.internet.reactor') as mock_reactor:
 
-            with patch('twisted.internet.reactor') as mock_reactor:
+                    protocol, client = AsyncModbusSerialClient(schedulers.REACTOR,
+                                                               method=method,
+                                                               port=SERIAL_PORT,
+                                                               proto_cls=ModbusSerClientProtocol)
 
-                protocol, client = AsyncModbusSerialClient(schedulers.REACTOR,
-                                                           method=method,
-                                                           port=SERIAL_PORT,
-                                                           proto_cls=ModbusSerClientProtocol)
+                    assert (isinstance(client, SerialPort))
+                    assert (isinstance(client.protocol, ModbusSerClientProtocol))
+                    assert (0 == len(list(client.protocol.transaction)))
+                    assert (isinstance(client.protocol.framer, framer))
+                    assert (client.protocol._connected)
 
-                assert (isinstance(client, SerialPort))
-                assert (isinstance(client.protocol, ModbusSerClientProtocol))
-                assert (0 == len(list(client.protocol.transaction)))
-                assert (isinstance(client.protocol.framer, framer))
-                assert (client.protocol._connected)
+                    def handle_failure(failure):
+                        assert (isinstance(failure.exception(), ConnectionException))
 
-                def handle_failure(failure):
-                    assert (isinstance(failure.exception(), ConnectionException))
+                    d = client.protocol._buildResponse(0x00)
+                    d.addCallback(handle_failure)
 
-                d = client.protocol._buildResponse(0x00)
-                d.addCallback(handle_failure)
-
-                assert (client.protocol._connected)
-                client.protocol.close()
-                protocol.stop()
-                assert (not client.protocol._connected)
+                    assert (client.protocol._connected)
+                    client.protocol.close()
+                    protocol.stop()
+                    assert (not client.protocol._connected)
 
     @pytest.mark.parametrize("method, framer", [("rtu", ModbusRtuFramer),
                                         ("socket", ModbusSocketFramer),
@@ -191,29 +236,31 @@ class TestAsynchronousClient(object):
                                         ("ascii", ModbusAsciiFramer)])
     def testSerialTornadoClient(self, method, framer):
         """ Test the serial tornado client client initialize """
-        protocol, future = AsyncModbusSerialClient(schedulers.IO_LOOP, method=method, port=SERIAL_PORT)
-        client = future.result()
-        assert(isinstance(client, AsyncTornadoModbusSerialClient))
-        assert(0 == len(list(client.transaction)))
-        assert(isinstance(client.framer, framer))
-        assert(client.port == SERIAL_PORT)
-        assert(client._connected)
+        from serial import Serial
+        with maybe_manage(sys.platform in ('darwin', 'win32'), patch.object(Serial, "open")):
+            protocol, future = AsyncModbusSerialClient(schedulers.IO_LOOP, method=method, port=SERIAL_PORT)
+            client = future.result()
+            assert(isinstance(client, AsyncTornadoModbusSerialClient))
+            assert(0 == len(list(client.transaction)))
+            assert(isinstance(client.framer, framer))
+            assert(client.port == SERIAL_PORT)
+            assert(client._connected)
 
-        def handle_failure(failure):
-            assert(isinstance(failure.exception(), ConnectionException))
+            def handle_failure(failure):
+                assert(isinstance(failure.exception(), ConnectionException))
 
-        d = client._build_response(0x00)
-        d.add_done_callback(handle_failure)
+            d = client._build_response(0x00)
+            d.add_done_callback(handle_failure)
 
-        assert(client._connected)
-        client.close()
-        protocol.stop()
-        assert(not client._connected)
+            assert(client._connected)
+            client.close()
+            protocol.stop()
+            assert(not client._connected)
 
     @pytest.mark.skipif(IS_PYTHON3 , reason="requires python2.7")
     def testSerialAsyncioClientPython2(self):
         """
-        Test Serial async asyncio client exits on python2
+        Test Serial asynchronous asyncio client exits on python2
         :return:
         """
         with pytest.raises(SystemExit) as pytest_wrapped_e:
@@ -234,6 +281,7 @@ class TestAsynchronousClient(object):
         :return:
         """
         loop = asyncio.get_event_loop()
+        loop.is_running.side_effect = lambda: False
         loop, client = AsyncModbusSerialClient(schedulers.ASYNC_IO, method=method, port=SERIAL_PORT, loop=loop,
                                                baudrate=19200, parity='E', stopbits=2, bytesize=7)
         assert(isinstance(client, AsyncioModbusSerialClient))
@@ -243,7 +291,8 @@ class TestAsynchronousClient(object):
         assert(client.parity == 'E')
         assert(client.stopbits == 2)
         assert(client.bytesize == 7)
-
+        client.stop()
+        loop.stop()
 
 # ---------------------------------------------------------------------------#
 # Main
