@@ -1,5 +1,7 @@
 #!/usr/bin/env python
 import unittest
+from itertools import count
+from io import StringIO
 from pymodbus.compat import IS_PYTHON3
 
 if IS_PYTHON3:  # Python 3
@@ -8,13 +10,20 @@ else:  # Python 2
     from mock import patch, Mock, MagicMock
 import socket
 import serial
+import ssl
+import sys
+
+import pytest
 
 from pymodbus.client.sync import ModbusTcpClient, ModbusUdpClient
 from pymodbus.client.sync import ModbusSerialClient, BaseModbusClient
+from pymodbus.client.sync import ModbusTlsClient
 from pymodbus.exceptions import ConnectionException, NotImplementedException
 from pymodbus.exceptions import ParameterException
 from pymodbus.transaction import ModbusAsciiFramer, ModbusRtuFramer
 from pymodbus.transaction import ModbusBinaryFramer
+from pymodbus.transaction import ModbusSocketFramer
+from pymodbus.utilities import hexlify_packets
 
 
 # ---------------------------------------------------------------------------#
@@ -41,6 +50,15 @@ class mockSocket(object):
     def in_waiting(self): return None
 
 
+inet_pton_skipif = pytest.mark.skipif(
+    sys.platform == "win32" and sys.version_info < (3, 4),
+    reason=(
+        "Uses socket.inet_pton() which wasn't available on Windows until"
+        " 3.4.",
+    )
+)
+
+
 
 # ---------------------------------------------------------------------------#
 # Fixture
@@ -60,8 +78,8 @@ class SynchronousClientTest(unittest.TestCase):
         client = BaseModbusClient(None)
         client.transaction = None
         self.assertRaises(NotImplementedException, lambda: client.connect())
-        self.assertRaises(NotImplementedException, lambda: client._send(None))
-        self.assertRaises(NotImplementedException, lambda: client._recv(None))
+        self.assertRaises(NotImplementedException, lambda: client.send(None))
+        self.assertRaises(NotImplementedException, lambda: client.recv(None))
         self.assertRaises(NotImplementedException, lambda: client.__enter__())
         self.assertRaises(NotImplementedException, lambda: client.execute())
         self.assertRaises(NotImplementedException, lambda: client.is_socket_open())
@@ -69,6 +87,20 @@ class SynchronousClientTest(unittest.TestCase):
         client.close()
         client.__exit__(0, 0, 0)
 
+        # Test information methods
+        client.last_frame_end = 2
+        client.silent_interval = 2
+        self.assertEqual(4, client.idle_time())
+        client.last_frame_end = None
+        self.assertEqual(0, client.idle_time())
+
+        # Test debug/trace/_dump methods
+        self.assertEqual(False, client.debug_enabled())
+        writable = StringIO()
+        client.trace(writable)
+        client._dump(b'\x00\x01\x02', None)
+        self.assertEqual(hexlify_packets(b'\x00\x01\x02'), writable.getvalue())
+
         # a successful execute
         client.connect = lambda: True
         client.transaction = Mock(**{'execute.return_value': True})
@@ -108,6 +140,7 @@ class SynchronousClientTest(unittest.TestCase):
 
         self.assertEqual("ModbusUdpClient(127.0.0.1:502)", str(client))
 
+    @inet_pton_skipif
     def testUdpClientAddressFamily(self):
         ''' Test the Udp client get address family method'''
         client = ModbusUdpClient()
@@ -115,6 +148,7 @@ class SynchronousClientTest(unittest.TestCase):
                          client._get_address_family('127.0.0.1'))
         self.assertEqual(socket.AF_INET6, client._get_address_family('::1'))
 
+    @inet_pton_skipif
     def testUdpClientConnect(self):
         ''' Test the Udp client connection method'''
         with patch.object(socket, 'socket') as mock_method:
@@ -131,6 +165,12 @@ class SynchronousClientTest(unittest.TestCase):
             client = ModbusUdpClient()
             self.assertFalse(client.connect())
 
+    @inet_pton_skipif
+    def testUdpClientIsSocketOpen(self):
+        ''' Test the udp client is_socket_open method'''
+        client = ModbusUdpClient()
+        self.assertTrue(client.is_socket_open())
+
     def testUdpClientSend(self):
         ''' Test the udp client send method'''
         client = ModbusUdpClient()
@@ -157,7 +197,6 @@ class SynchronousClientTest(unittest.TestCase):
         )
         self.assertEqual(repr(client), rep)
 
-
     # -----------------------------------------------------------------------#
     # Test TCP Client
     # -----------------------------------------------------------------------#
@@ -191,8 +230,10 @@ class SynchronousClientTest(unittest.TestCase):
     def testTcpClientConnect(self):
         ''' Test the tcp client connection method'''
         with patch.object(socket, 'create_connection') as mock_method:
-            mock_method.return_value = object()
+            _socket = MagicMock()
+            mock_method.return_value = _socket
             client = ModbusTcpClient()
+            _socket.getsockname.return_value = ('dmmy', 1234)
             self.assertTrue(client.connect())
 
         with patch.object(socket, 'create_connection') as mock_method:
@@ -200,6 +241,11 @@ class SynchronousClientTest(unittest.TestCase):
             client = ModbusTcpClient()
             self.assertFalse(client.connect())
 
+    def testTcpClientIsSocketOpen(self):
+        ''' Test the tcp client is_socket_open method'''
+        client = ModbusTcpClient()
+        self.assertFalse(client.is_socket_open())
+
     def testTcpClientSend(self):
         ''' Test the tcp client send method'''
         client = ModbusTcpClient()
@@ -209,11 +255,13 @@ class SynchronousClientTest(unittest.TestCase):
         self.assertEqual(0, client._send(None))
         self.assertEqual(4, client._send('1234'))
 
+    @patch('pymodbus.client.sync.time')
     @patch('pymodbus.client.sync.select')
-    def testTcpClientRecv(self, mock_select):
+    def testTcpClientRecv(self, mock_select, mock_time):
         ''' Test the tcp client receive method'''
 
         mock_select.select.return_value = [True]
+        mock_time.time.side_effect = count()
         client = ModbusTcpClient()
         self.assertRaises(ConnectionException, lambda: client._recv(1024))
 
@@ -224,7 +272,7 @@ class SynchronousClientTest(unittest.TestCase):
         mock_socket = MagicMock()
         mock_socket.recv.side_effect = iter([b'\x00', b'\x01', b'\x02'])
         client.socket = mock_socket
-        client.timeout = 1
+        client.timeout = 3
         self.assertEqual(b'\x00\x01\x02', client._recv(3))
         mock_socket.recv.side_effect = iter([b'\x00', b'\x01', b'\x02'])
         self.assertEqual(b'\x00\x01', client._recv(2))
@@ -234,13 +282,127 @@ class SynchronousClientTest(unittest.TestCase):
         mock_select.select.return_value = [True]
         self.assertIn(b'\x00', client._recv(None))
 
-    def testSerialClientRpr(self):
+        mock_socket = MagicMock()
+        mock_socket.recv.return_value = b''
+        client.socket = mock_socket
+        self.assertRaises(ConnectionException, lambda: client._recv(1024))
+
+        mock_socket.recv.side_effect = iter([b'\x00', b'\x01', b'\x02', b''])
+        client.socket = mock_socket
+        self.assertEqual(b'\x00\x01\x02', client._recv(1024))
+
+    def testTcpClientRpr(self):
         client = ModbusTcpClient()
         rep = "<{} at {} socket={}, ipaddr={}, port={}, timeout={}>".format(
             client.__class__.__name__, hex(id(client)), client.socket,
             client.host, client.port, client.timeout
         )
         self.assertEqual(repr(client), rep)
+
+    def testTcpClientRegister(self):
+        class CustomeRequest:
+            function_code = 79
+        client = ModbusTcpClient()
+        client.framer = Mock()
+        client.register(CustomeRequest)
+        assert client.framer.decoder.register.called_once_with(CustomeRequest)
+
+    # -----------------------------------------------------------------------#
+    # Test TLS Client
+    # -----------------------------------------------------------------------#
+
+    def testSyncTlsClientInstantiation(self):
+        # default SSLContext
+        client = ModbusTlsClient()
+        self.assertNotEqual(client, None)
+        self.assertTrue(client.sslctx)
+
+        # user defined SSLContext
+        context = ssl.create_default_context()
+        client = ModbusTlsClient(sslctx=context)
+        self.assertNotEqual(client, None)
+        self.assertEqual(client.sslctx, context)
+
+    def testBasicSyncTlsClient(self):
+        ''' Test the basic methods for the tls sync client'''
+
+        # receive/send
+        client = ModbusTlsClient()
+        client.socket = mockSocket()
+        self.assertEqual(0, client._send(None))
+        self.assertEqual(1, client._send(b'\x00'))
+        self.assertEqual(b'\x00', client._recv(1))
+
+        # connect/disconnect
+        self.assertTrue(client.connect())
+        client.close()
+
+        # already closed socket
+        client.socket = False
+        client.close()
+
+        self.assertEqual("ModbusTlsClient(localhost:802)", str(client))
+
+    def testTlsClientConnect(self):
+        ''' Test the tls client connection method'''
+        with patch.object(ssl.SSLSocket, 'connect') as mock_method:
+            client = ModbusTlsClient()
+            self.assertTrue(client.connect())
+
+        with patch.object(socket, 'create_connection') as mock_method:
+            mock_method.side_effect = socket.error()
+            client = ModbusTlsClient()
+            self.assertFalse(client.connect())
+
+    def testTlsClientSend(self):
+        ''' Test the tls client send method'''
+        client = ModbusTlsClient()
+        self.assertRaises(ConnectionException, lambda: client._send(None))
+
+        client.socket = mockSocket()
+        self.assertEqual(0, client._send(None))
+        self.assertEqual(4, client._send('1234'))
+
+    @patch('pymodbus.client.sync.time')
+    def testTlsClientRecv(self, mock_time):
+        ''' Test the tls client receive method'''
+        client = ModbusTlsClient()
+        self.assertRaises(ConnectionException, lambda: client._recv(1024))
+
+        mock_time.time.side_effect = count()
+
+        client.socket = mockSocket()
+        self.assertEqual(b'', client._recv(0))
+        self.assertEqual(b'\x00' * 4, client._recv(4))
+
+        client.timeout = 2
+        self.assertIn(b'\x00', client._recv(None))
+
+        mock_socket = MagicMock()
+        mock_socket.recv.side_effect = iter([b'\x00', b'\x01', b'\x02'])
+        client.socket = mock_socket
+        client.timeout = 3
+        self.assertEqual(b'\x00\x01\x02', client._recv(3))
+        mock_socket.recv.side_effect = iter([b'\x00', b'\x01', b'\x02'])
+        self.assertEqual(b'\x00\x01', client._recv(2))
+
+    def testTlsClientRpr(self):
+        client = ModbusTlsClient()
+        rep = "<{} at {} socket={}, ipaddr={}, port={}, sslctx={}, " \
+            "timeout={}>".format(
+            client.__class__.__name__, hex(id(client)), client.socket,
+            client.host, client.port, client.sslctx, client.timeout
+        )
+        self.assertEqual(repr(client), rep)
+
+    def testTlsClientRegister(self):
+        class CustomeRequest:
+            function_code = 79
+        client = ModbusTlsClient()
+        client.framer = Mock()
+        client.register(CustomeRequest)
+        assert client.framer.decoder.register.called_once_with(CustomeRequest)
+
     # -----------------------------------------------------------------------#
     # Test Serial Client
     # -----------------------------------------------------------------------#
@@ -254,6 +416,8 @@ class SynchronousClientTest(unittest.TestCase):
                                    ModbusRtuFramer))
         self.assertTrue(isinstance(ModbusSerialClient(method='binary').framer,
                                    ModbusBinaryFramer))
+        self.assertTrue(isinstance(ModbusSerialClient(method='socket').framer,
+                                   ModbusSocketFramer))
         self.assertRaises(ParameterException,
                           lambda: ModbusSerialClient(method='something'))
 
@@ -284,6 +448,12 @@ class SynchronousClientTest(unittest.TestCase):
         self.assertTrue(client.connect())
         client.close()
 
+        # rtu connect/disconnect
+        rtu_client = ModbusSerialClient(method='rtu', strict=True)
+        self.assertTrue(rtu_client.connect())
+        self.assertEqual(rtu_client.socket.interCharTimeout, rtu_client.inter_char_timeout)
+        rtu_client.close()
+
         # already closed socket
         client.socket = False
         client.close()
@@ -293,7 +463,7 @@ class SynchronousClientTest(unittest.TestCase):
     def testSerialClientConnect(self):
         ''' Test the serial client connection method'''
         with patch.object(serial, 'Serial') as mock_method:
-            mock_method.return_value = object()
+            mock_method.return_value = MagicMock()
             client = ModbusSerialClient()
             self.assertTrue(client.connect())
 
@@ -302,6 +472,14 @@ class SynchronousClientTest(unittest.TestCase):
             client = ModbusSerialClient()
             self.assertFalse(client.connect())
 
+    @patch("serial.Serial")
+    def testSerialClientIsSocketOpen(self, mock_serial):
+        ''' Test the serial client is_socket_open method'''
+        client = ModbusSerialClient()
+        self.assertFalse(client.is_socket_open())
+        client.socket = mock_serial
+        self.assertTrue(client.is_socket_open())
+
     @patch("serial.Serial")
     def testSerialClientSend(self, mock_serial):
         ''' Test the serial client send method'''
@@ -344,6 +522,8 @@ class SynchronousClientTest(unittest.TestCase):
         self.assertEqual(b'', client._recv(None))
         client.socket.timeout = 0
         self.assertEqual(b'', client._recv(0))
+        client.timeout = None
+        self.assertEqual(b'', client._recv(None))
 
     def testSerialClientRepr(self):
         client = ModbusSerialClient()
@@ -352,6 +532,8 @@ class SynchronousClientTest(unittest.TestCase):
             client.method, client.timeout
         )
         self.assertEqual(repr(client), rep)
+
+
 # ---------------------------------------------------------------------------#
 # Main
 # ---------------------------------------------------------------------------#
