@@ -98,9 +98,44 @@ class ModbusDataStoreTest(unittest.TestCase):
         block.setValues(0x00, dict(enumerate([False]*10)))
         self.assertEqual(block.getValues(0x00, 10), [False]*10)
 
+        block = ModbusSparseDataBlock({3: [10, 11, 12], 10: 1, 15: [0] * 4})
+        self.assertEqual(block.values, {3: 10, 4: 11, 5: 12, 10: 1,
+                                        15:0 , 16:0, 17:0, 18:0 })
+        self.assertEqual(block.default_value, {3: 10, 4: 11, 5: 12, 10: 1,
+                                        15:0 , 16:0, 17:0, 18:0 })
+        self.assertEqual(block.mutable, True)
+        block.setValues(3, [20, 21, 22, 23], use_as_default=True)
+        self.assertEqual(block.getValues(3, 4), [20, 21, 22, 23])
+        self.assertEqual(block.default_value, {3: 20, 4: 21, 5: 22, 6:23, 10: 1,
+                                        15:0 , 16:0, 17:0, 18:0 })
+        # check when values is a dict, address is ignored
+        block.setValues(0, {5: 32, 7: 43})
+        self.assertEqual(block.getValues(5, 3), [32, 23, 43])
+
+        # assert value is empty dict when initialized without params
+        block = ModbusSparseDataBlock()
+        self.assertEqual(block.values, {})
+
+        # mark block as unmutable and see if parameter exeception
+        # is raised for invalid offset writes
+        block = ModbusSparseDataBlock({1: 100}, mutable=False)
+        self.assertRaises(ParameterException, block.setValues, 0, 1)
+        self.assertRaises(ParameterException, block.setValues, 0, {2: 100})
+        self.assertRaises(ParameterException, block.setValues, 0, [1] * 10)
+
+        # Reset datablock
+        block = ModbusSparseDataBlock({3: [10, 11, 12], 10: 1, 15: [0] * 4})
+        block.setValues(0, {3: [20, 21, 22], 10: 11, 15: [10] * 4})
+        self.assertEqual(block.values, {3: 20, 4: 21, 5: 22, 10: 11,
+                                        15: 10 ,16:10, 17:10, 18:10 })
+        block.reset()
+        self.assertEqual(block.values, {3: 10, 4: 11, 5: 12, 10: 1,
+                                        15: 0, 16: 0, 17: 0, 18: 0})
+
+
     def testModbusSparseDataBlockFactory(self):
         ''' Test the sparse data block store factory '''
-        block = ModbusSparseDataBlock.create()
+        block = ModbusSparseDataBlock.create([0x00]*65536)
         self.assertEqual(block.getValues(0x00, 65536), [False]*65536)
 
     def testModbusSparseDataBlockOther(self):
@@ -109,6 +144,7 @@ class ModbusDataStoreTest(unittest.TestCase):
         self.assertRaises(ParameterException,
             lambda: ModbusSparseDataBlock(True))
 
+
     def testModbusSlaveContext(self):
         ''' Test a modbus slave context '''
         store = {
@@ -278,19 +314,16 @@ class SqlDataStoreTest(unittest.TestCase):
         self.slave._db_create.assert_called_once_with(
             self.slave.table, self.slave.database
         )
+
     def testValidateSuccess(self):
-        mock_result = MockSqlResult(
-            rowcount=len(self.mock_values)
-        )
-        self.slave._connection.execute = MagicMock(return_value=mock_result)
+        self.slave._connection.execute.return_value.fetchall.return_value = self.mock_values
         self.assertTrue(self.slave.validate(
             self.mock_function, self.mock_addr, len(self.mock_values))
         )
 
     def testValidateFailure(self):
         wrong_count = 9
-        mock_result = MockSqlResult(rowcount=len(self.mock_values))
-        self.slave._connection.execute = MagicMock(return_value=mock_result)
+        self.slave._connection.execute.return_value.fetchall.return_value = self.mock_values
         self.assertFalse(self.slave.validate(
             self.mock_function, self.mock_addr, wrong_count)
         )
@@ -333,7 +366,8 @@ class SqlDataStoreTest(unittest.TestCase):
         self.slave._set = MagicMock()
 
         for key, value in self.function_map.items():
-            self.slave.setValues(key, self.mock_addr, self.mock_values)
+            self.slave.setValues(key, self.mock_addr,
+                                 self.mock_values, update=False)
             self.slave._set.assert_called_with(
                 value, self.mock_addr + 1, self.mock_values
             )
@@ -368,8 +402,9 @@ class SqlDataStoreTest(unittest.TestCase):
             self.slave._update(self.mock_type, self.mock_offset, self.mock_values)
         )
 
-#---------------------------------------------------------------------------#
+
+# --------------------------------------------------------------------------- #
 # Main
-#---------------------------------------------------------------------------#
+# --------------------------------------------------------------------------- #
 if __name__ == "__main__":
     unittest.main()
