@@ -6,14 +6,6 @@ class ModbusFixesTest(unittest.TestCase):
     This is the unittest for the pymodbus._version code
     '''
 
-    def setUp(self):
-        ''' Initializes the test environment '''
-        pass
-
-    def tearDown(self):
-        ''' Cleans up the test environment '''
-        pass
-
     def testTrueFalseDefined(self):
         ''' Test that True and False are defined on all versions'''
         try:
@@ -26,9 +18,8 @@ class ModbusFixesTest(unittest.TestCase):
     def testNullLoggerAttached(self):
         ''' Test that the null logger is attached'''
         import logging
-        if len(logging._handlers) == 0:
-          import pymodbus
-          self.assertEqual(logging._handlers, 1)
+        logger = logging.getLogger('pymodbus')
+        self.assertEqual(len(logger.handlers), 1)
 
 #---------------------------------------------------------------------------#
 # Main
