@@ -7,14 +7,15 @@ else: # Python 2
     from mock import patch, Mock
 import serial
 import socket
+import ssl
 
 from pymodbus.device import ModbusDeviceIdentification
 from pymodbus.server.sync import ModbusBaseRequestHandler
 from pymodbus.server.sync import ModbusSingleRequestHandler
 from pymodbus.server.sync import ModbusConnectedRequestHandler
 from pymodbus.server.sync import ModbusDisconnectedRequestHandler
-from pymodbus.server.sync import ModbusTcpServer, ModbusUdpServer, ModbusSerialServer
-from pymodbus.server.sync import StartTcpServer, StartUdpServer, StartSerialServer
+from pymodbus.server.sync import ModbusTcpServer, ModbusTlsServer, ModbusUdpServer, ModbusSerialServer
+from pymodbus.server.sync import StartTcpServer, StartTlsServer, StartUdpServer, StartSerialServer
 from pymodbus.exceptions import NotImplementedException
 from pymodbus.bit_read_message import ReadCoilsRequest, ReadCoilsResponse
 from pymodbus.datastore import ModbusServerContext
@@ -259,13 +260,12 @@ class SynchronousServerTest(unittest.TestCase):
     #-----------------------------------------------------------------------#
     def testTcpServerClose(self):
         ''' test that the synchronous TCP server closes correctly '''
-        with patch.object(socket.socket, 'bind') as mock_socket:
-            identity = ModbusDeviceIdentification(info={0x00: 'VendorName'})
-            server = ModbusTcpServer(context=None, identity=identity)
-            server.threads.append(Mock(**{'running': True}))
-            server.server_close()
-            self.assertEqual(server.control.Identity.VendorName, 'VendorName')
-            self.assertFalse(server.threads[0].running)
+        identity = ModbusDeviceIdentification(info={0x00: 'VendorName'})
+        server = ModbusTcpServer(context=None, identity=identity, bind_and_activate=False)
+        server.threads.append(Mock(**{'running': True}))
+        server.server_close()
+        self.assertEqual(server.control.Identity.VendorName, 'VendorName')
+        self.assertFalse(server.threads[0].running)
 
     def testTcpServerProcess(self):
         ''' test that the synchronous TCP server processes requests '''
@@ -275,18 +275,60 @@ class SynchronousServerTest(unittest.TestCase):
             self.assertTrue(mock_server.process_request.called)
 
     #-----------------------------------------------------------------------#
-    # Test UDP Server
+    # Test TLS Server
     #-----------------------------------------------------------------------#
-    def testUdpServerClose(self):
-        ''' test that the synchronous UDP server closes correctly '''
-        with patch.object(socket.socket, 'bind') as mock_socket:
+    def testTlsServerInit(self):
+        ''' test that the synchronous TLS server initial correctly '''
+        with patch.object(socketserver.TCPServer, 'server_activate'):
+            with patch.object(ssl.SSLContext, 'load_cert_chain') as mock_method:
+                identity = ModbusDeviceIdentification(info={0x00: 'VendorName'})
+                server = ModbusTlsServer(context=None, identity=identity,
+                                         bind_and_activate=False)
+                server.server_activate()
+                self.assertIsNotNone(server.sslctx)
+                self.assertEqual(type(server.socket), ssl.SSLSocket)
+                server.server_close()
+                sslctx = ssl.create_default_context()
+                server = ModbusTlsServer(context=None, identity=identity,
+                                         sslctx=sslctx, bind_and_activate=False)
+                server.server_activate()
+                self.assertEqual(server.sslctx, sslctx)
+                self.assertEqual(type(server.socket), ssl.SSLSocket)
+                server.server_close()
+
+    def testTlsServerClose(self):
+        ''' test that the synchronous TLS server closes correctly '''
+        with patch.object(ssl.SSLContext, 'load_cert_chain') as mock_method:
             identity = ModbusDeviceIdentification(info={0x00: 'VendorName'})
-            server = ModbusUdpServer(context=None, identity=identity)
+            server = ModbusTlsServer(context=None, identity=identity,
+                                     bind_and_activate=False)
             server.threads.append(Mock(**{'running': True}))
             server.server_close()
             self.assertEqual(server.control.Identity.VendorName, 'VendorName')
             self.assertFalse(server.threads[0].running)
 
+    def testTlsServerProcess(self):
+        ''' test that the synchronous TLS server processes requests '''
+        with patch('pymodbus.compat.socketserver.ThreadingTCPServer') as mock_server:
+            with patch.object(ssl.SSLContext, 'load_cert_chain') as mock_method:
+                server = ModbusTlsServer(None)
+                server.process_request('request', 'client')
+                self.assertTrue(mock_server.process_request.called)
+
+    #-----------------------------------------------------------------------#
+    # Test UDP Server
+    #-----------------------------------------------------------------------#
+    def testUdpServerClose(self):
+        ''' test that the synchronous UDP server closes correctly '''
+        identity = ModbusDeviceIdentification(info={0x00: 'VendorName'})
+        server = ModbusUdpServer(context=None, identity=identity,
+                                 bind_and_activate=False)
+        server.server_activate()
+        server.threads.append(Mock(**{'running': True}))
+        server.server_close()
+        self.assertEqual(server.control.Identity.VendorName, 'VendorName')
+        self.assertFalse(server.threads[0].running)
+
     def testUdpServerProcess(self):
         ''' test that the synchronous UDP server processes requests '''
         with patch('pymodbus.compat.socketserver.ThreadingUDPServer') as mock_server:
@@ -326,9 +368,9 @@ class SynchronousServerTest(unittest.TestCase):
             with patch('pymodbus.server.sync.CustomSingleRequestHandler') as mock_handler:
                 server = ModbusSerialServer(None)
                 instance = mock_handler.return_value
-                instance.handle.side_effect = server.server_close
+                instance.response_manipulator.side_effect = server.server_close
                 server.serve_forever()
-                instance.handle.assert_any_call()
+                instance.response_manipulator.assert_any_call()
 
     def testSerialServerClose(self):
         ''' test that the synchronous serial server closes correctly '''
@@ -344,8 +386,13 @@ class SynchronousServerTest(unittest.TestCase):
     def testStartTcpServer(self):
         ''' Test the tcp server starting factory '''
         with patch.object(ModbusTcpServer, 'serve_forever') as mock_server:
-            with patch.object(socketserver.TCPServer, 'server_bind') as mock_binder:
-                StartTcpServer()
+            StartTcpServer(bind_and_activate=False)
+
+    def testStartTlsServer(self):
+        ''' Test the tls server starting factory '''
+        with patch.object(ModbusTlsServer, 'serve_forever') as mock_server:
+            with patch.object(ssl.SSLContext, 'load_cert_chain') as mock_method:
+                StartTlsServer(bind_and_activate=False)
 
     def testStartUdpServer(self):
         ''' Test the udp server starting factory '''
