@@ -1,12 +1,20 @@
 #!/usr/bin/env python
 import pytest
 import unittest
+from itertools import count
+from pymodbus.compat import IS_PYTHON3
+
+if IS_PYTHON3:  # Python 3
+    from unittest.mock import patch, Mock, MagicMock
+else:  # Python 2
+    from mock import patch, Mock, MagicMock
+
 from binascii import a2b_hex
 from pymodbus.pdu import *
 from pymodbus.transaction import *
 from pymodbus.transaction import (
-    ModbusTransactionManager, ModbusSocketFramer, ModbusAsciiFramer,
-    ModbusRtuFramer, ModbusBinaryFramer
+    ModbusTransactionManager, ModbusSocketFramer, ModbusTlsFramer,
+    ModbusAsciiFramer, ModbusRtuFramer, ModbusBinaryFramer
 )
 from pymodbus.factory import ServerDecoder
 from pymodbus.compat import byte2int
@@ -29,6 +37,7 @@ class ModbusTransactionTest(unittest.TestCase):
         self.client   = None
         self.decoder  = ServerDecoder()
         self._tcp     = ModbusSocketFramer(decoder=self.decoder, client=None)
+        self._tls     = ModbusTlsFramer(decoder=self.decoder, client=None)
         self._rtu     = ModbusRtuFramer(decoder=self.decoder, client=None)
         self._ascii   = ModbusAsciiFramer(decoder=self.decoder, client=None)
         self._binary  = ModbusBinaryFramer(decoder=self.decoder, client=None)
@@ -40,6 +49,7 @@ class ModbusTransactionTest(unittest.TestCase):
         """ Cleans up the test environment """
         del self._manager
         del self._tcp
+        del self._tls
         del self._rtu
         del self._ascii
 
@@ -60,6 +70,7 @@ class ModbusTransactionTest(unittest.TestCase):
                                          ('binary', 7),
                                          ('rtu', 5),
                                          ('tcp', 9),
+                                         ('tls', 2),
                                          ('dummy', None)]:
             self._tm.client = MagicMock()
             if framer == "ascii":
@@ -70,6 +81,8 @@ class ModbusTransactionTest(unittest.TestCase):
                 self._tm.client.framer = self._rtu
             elif framer == "tcp":
                 self._tm.client.framer = self._tcp
+            elif framer == "tls":
+                self._tm.client.framer = self._tls
             else:
                 self._tm.client.framer = MagicMock()
 
@@ -77,7 +90,10 @@ class ModbusTransactionTest(unittest.TestCase):
             self.assertEqual(self._tm._calculate_exception_length(),
                              exception_length)
 
-    def testExecute(self):
+    @patch('pymodbus.transaction.time')
+    def testExecute(self, mock_time):
+        mock_time.time.side_effect = count()
+
         client = MagicMock()
         client.framer = self._ascii
         client.framer._buffer = b'deadbeef'
@@ -87,10 +103,16 @@ class ModbusTransactionTest(unittest.TestCase):
         client.framer.buildPacket.return_value = b'deadbeef'
         client.framer.sendPacket = MagicMock()
         client.framer.sendPacket.return_value = len(b'deadbeef')
-
+        client.framer.decode_data = MagicMock()
+        client.framer.decode_data.return_value = {
+            "unit": 1,
+            "fcode": 222,
+            "length": 27
+        }
         request = MagicMock()
         request.get_response_pdu_size.return_value = 10
         request.unit_id = 1
+        request.function_code = 222
         tm = ModbusTransactionManager(client)
         tm._recv = MagicMock(return_value=b'abcdef')
         self.assertEqual(tm.retries, 3)
@@ -98,6 +120,7 @@ class ModbusTransactionTest(unittest.TestCase):
         # tm._transact = MagicMock()
         # some response
         # tm._transact.return_value = (b'abcdef', None)
+
         tm.getTransaction = MagicMock()
         tm.getTransaction.return_value = 'response'
         response = tm.execute(request)
@@ -118,12 +141,36 @@ class ModbusTransactionTest(unittest.TestCase):
         response = tm.execute(request)
         self.assertIsInstance(response, ModbusIOException)
 
+        # wrong handle_local_echo
+        tm._recv = MagicMock(side_effect=iter([b'abcdef', b'deadbe', b'123456']))
+        client.handle_local_echo = True
+        tm.retry_on_empty = False
+        tm.retry_on_invalid = False
+        self.assertEqual(tm.execute(request).message,
+                         '[Input/Output] Wrong local echo')
+        client.handle_local_echo = False
+
+        # retry on invalid response
+        tm.retry_on_invalid = True
+        tm._recv = MagicMock(side_effect=iter([b'', b'abcdef', b'deadbe', b'123456']))
+        # tm._transact.side_effect = [(b'', None), (b'abcdef', None)]
+        response = tm.execute(request)
+        self.assertIsInstance(response, ModbusIOException)
+
         # Unable to decode response
         tm._recv = MagicMock(side_effect=ModbusIOException())
         # tm._transact.side_effect = [(b'abcdef', None)]
         client.framer.processIncomingPacket.side_effect = MagicMock(side_effect=ModbusIOException())
         self.assertIsInstance(tm.execute(request), ModbusIOException)
 
+        # Broadcast
+        client.broadcast_enable = True
+        request.unit_id = 0
+        response = tm.execute(request)
+        self.assertEqual(response, b'Broadcast write sent - '
+                                   b'no response expected')
+
+
     # ----------------------------------------------------------------------- #
     # Dictionary based transaction manager
     # ----------------------------------------------------------------------- #
@@ -303,6 +350,137 @@ class ModbusTransactionTest(unittest.TestCase):
         self.assertEqual(expected, actual)
         ModbusRequest.encode = old_encode
 
+    # ----------------------------------------------------------------------- #
+    # TLS tests
+    # ----------------------------------------------------------------------- #
+    def testTLSFramerTransactionReady(self):
+        """ Test a tls frame transaction """
+        msg = b"\x01\x12\x34\x00\x08"
+        self.assertFalse(self._tls.isFrameReady())
+        self.assertFalse(self._tls.checkFrame())
+        self._tls.addToFrame(msg)
+        self.assertTrue(self._tls.isFrameReady())
+        self.assertTrue(self._tls.checkFrame())
+        self._tls.advanceFrame()
+        self.assertFalse(self._tls.isFrameReady())
+        self.assertFalse(self._tls.checkFrame())
+        self.assertEqual(b'', self._tls.getFrame())
+
+    def testTLSFramerTransactionFull(self):
+        """ Test a full tls frame transaction """
+        msg = b"\x01\x12\x34\x00\x08"
+        self._tls.addToFrame(msg)
+        self.assertTrue(self._tls.checkFrame())
+        result = self._tls.getFrame()
+        self.assertEqual(msg[0:], result)
+        self._tls.advanceFrame()
+
+    def testTLSFramerTransactionHalf(self):
+        """ Test a half completed tls frame transaction """
+        msg1 = b""
+        msg2 = b"\x01\x12\x34\x00\x08"
+        self._tls.addToFrame(msg1)
+        self.assertFalse(self._tls.checkFrame())
+        result = self._tls.getFrame()
+        self.assertEqual(b'', result)
+        self._tls.addToFrame(msg2)
+        self.assertTrue(self._tls.checkFrame())
+        result = self._tls.getFrame()
+        self.assertEqual(msg2[0:], result)
+        self._tls.advanceFrame()
+
+    def testTLSFramerTransactionShort(self):
+        """ Test that we can get back on track after an invalid message """
+        msg1 = b""
+        msg2 = b"\x01\x12\x34\x00\x08"
+        self._tls.addToFrame(msg1)
+        self.assertFalse(self._tls.checkFrame())
+        result = self._tls.getFrame()
+        self.assertEqual(b'', result)
+        self._tls.advanceFrame()
+        self._tls.addToFrame(msg2)
+        self.assertEqual(5, len(self._tls._buffer))
+        self.assertTrue(self._tls.checkFrame())
+        result = self._tls.getFrame()
+        self.assertEqual(msg2[0:], result)
+        self._tls.advanceFrame()
+
+    def testTLSFramerDecode(self):
+        """ Testmessage decoding """
+        msg1 = b""
+        msg2 = b"\x01\x12\x34\x00\x08"
+        result = self._tls.decode_data(msg1)
+        self.assertEqual(dict(), result);
+        result = self._tls.decode_data(msg2)
+        self.assertEqual(dict(fcode=1), result);
+        self._tls.advanceFrame()
+
+    def testTLSIncomingPacket(self):
+        msg = b"\x01\x12\x34\x00\x08"
+
+        unit = 0x01
+        def mock_callback(self):
+            pass
+
+        self._tls._process = MagicMock()
+        self._tls.isFrameReady = MagicMock(return_value=False)
+        self._tls.processIncomingPacket(msg, mock_callback, unit)
+        self.assertEqual(msg, self._tls.getRawFrame())
+        self._tls.advanceFrame()
+
+        self._tls.isFrameReady = MagicMock(return_value=True)
+        self._tls._validate_unit_id = MagicMock(return_value=False)
+        self._tls.processIncomingPacket(msg, mock_callback, unit)
+        self.assertEqual(b'', self._tls.getRawFrame())
+        self._tls.advanceFrame()
+
+        self._tls._validate_unit_id = MagicMock(return_value=True)
+        self._tls.processIncomingPacket(msg, mock_callback, unit)
+        self.assertEqual(msg, self._tls.getRawFrame())
+        self._tls.advanceFrame()
+
+    def testTLSProcess(self):
+        class MockResult(object):
+            def __init__(self, code):
+                self.function_code = code
+
+        def mock_callback(self):
+            pass
+
+        self._tls.decoder.decode = MagicMock(return_value=None)
+        self.assertRaises(ModbusIOException,
+                          lambda: self._tls._process(mock_callback))
+
+        result = MockResult(0x01)
+        self._tls.decoder.decode = MagicMock(return_value=result)
+        self.assertRaises(InvalidMessageReceivedException,
+                          lambda: self._tls._process(mock_callback, error=True))
+
+        self._tls._process(mock_callback)
+        self.assertEqual(b'', self._tls.getRawFrame())
+
+    def testTLSFramerPopulate(self):
+        """ Test a tls frame packet build """
+        expected = ModbusRequest()
+        msg = b"\x01\x12\x34\x00\x08"
+        self._tls.addToFrame(msg)
+        self.assertTrue(self._tls.checkFrame())
+        actual = ModbusRequest()
+        result = self._tls.populateResult(actual)
+        self.assertEqual(None, result)
+        self._tls.advanceFrame()
+
+    def testTLSFramerPacket(self):
+        """ Test a tls frame packet build """
+        old_encode = ModbusRequest.encode
+        ModbusRequest.encode = lambda self: b''
+        message = ModbusRequest()
+        message.function_code  = 0x01
+        expected = b"\x01"
+        actual = self._tls.buildPacket(message)
+        self.assertEqual(expected, actual)
+        ModbusRequest.encode = old_encode
+
     # ----------------------------------------------------------------------- #
     # RTU tests
     # ----------------------------------------------------------------------- #
@@ -312,7 +490,7 @@ class ModbusTransactionTest(unittest.TestCase):
 
         msg_parts = [b"\x00\x01\x00", b"\x00\x00\x01\xfc\x1b"]
         self._rtu.addToFrame(msg_parts[0])
-        self.assertTrue(self._rtu.isFrameReady())
+        self.assertFalse(self._rtu.isFrameReady())
         self.assertFalse(self._rtu.checkFrame())
 
         self._rtu.addToFrame(msg_parts[1])
